rawset(_G, "bounce", true)

freeslot("SKINCOLOR_SKYFARER")
skincolors[SKINCOLOR_SKYFARER] = {
	name = "Skyfarer",
	ramp = {32,192,163,149,150,151,153,154,168,169,159,159,154,159,31,31},
	invcolor = SKINCOLOR_RUBY,
	invshade = 5, 
	chatcolor = V_AZUREMAP,
	accessible = true
}

local function RIU_DefaultThrustFactor(p)
	local t = skins[p.mo.skin].thrustfactor
	if p.powers[pw_sneakers] or p.powers[pw_super] then t = (t*FRACUNIT/16*17)/FRACUNIT end
	return t
end

local function RIU_DefaultAccelStart(p)
	local t = skins[p.mo.skin].accelstart
	if p.powers[pw_sneakers] or p.powers[pw_super] then t = (t*FRACUNIT/16*17)/FRACUNIT end
	return t
end

local function RIU_DefaultAcceleration(p)
	local t = skins[p.mo.skin].acceleration
	if p.powers[pw_sneakers] or p.powers[pw_super] then t = (t*FRACUNIT/16*17)/FRACUNIT end
	return t
end

addHook("ThinkFrame", do
	for player in players.iterate
		//Eliminate possibilitys of NOT using A spring.
		//Obviously, since this is a workaround, it's far from perfect.
		if not P_IsObjectOnGround(player.mo)
		and not P_InQuicksand(player.mo)
		and not P_PlayerInPain(player)
		and not P_CheckDeathPitCollide(player.mo)
		and not (player.pflags & PF_SLIDING)	
		and not (player.pflags & PF_SPINNING)
		and not (player.pflags & PF_JUMPED)
		and not (player.powers[pw_carry])
		and not (player.skidtime > 0)
		and not (player.exiting)
		and not (player.gotflag)
		and not (player.pflags & PF_THOKKED)
		then
		//check for inversion, and assume player used a spring.
			if (player.mo.eflags & MFE_VERTICALFLIP)
				if player.mo.momz < -14*FRACUNIT
					player.springing = 1
				end
			else
				if player.mo.momz > 14*FRACUNIT
					player.springing = 1
				end
			end
		end
		//dissable on land, or other.
		if P_IsObjectOnGround(player.mo)
		or P_PlayerInPain(player)
		or P_CheckDeathPitCollide(player.mo)
		or (player.powers[pw_carry])
			player.springing = 0
		end	
	end
end)

addHook("PlayerThink", function(p)
	if not p.riu then
		p.riu = {
			wallhitcount = 0,
			airtime = 0,
			jumpboost = 0,
			jumpboostamount = 4,
			SpinUpFactor = 1,
			WHPrep = 0,
			savedx = 0,
			savedy = 0,
			savedz = 0,
			wallhit = 0,
			marbledived = 0
		}
	end
		
	if P_IsObjectOnGround(p.mo) then
		p.riu.wallhitcount = 2
		p.riu.jumpboost = 0
		p.riu.airtime = 0
		p.riu.spinwind = 0
	else
		if p.mo.state == S_PLAY_ROLL
			p.mo.state = S_PLAY_JUMP
		end
		p.riu.airtime = $ + 1
	end
	
	if not p.mo then return end
	
	p.thrustfactor = max(RIU_DefaultThrustFactor(p) + (40 - (p.speed/FRACUNIT))/10, 5)
	p.accelstart = (RIU_DefaultAccelStart(p) / 4) * 3
	p.acceleration = (RIU_DefaultAcceleration(p) / 4) * 3
	
	if p.riu.WHPrep < 0 then
		p.riu.WHPrep = $+100/3
	end
	
	if p.riu.WHPrep >= -1000 and p.riu.WHPrep <= -900 then
		p.mo.momx = 0
		p.mo.momy = 0
		p.mo.momz = 0
	end
	
	if p.riu.wallhit == 1 and p.riu.WHPrep >= -900 then
	
		p.mo.momx = p.riu.savedx
		p.mo.momy = p.riu.savedy
		p.mo.momz = p.riu.savedz
		
		p.pflags = ($ | PF_JUMPED) & ~(PF_SPINNING|PF_THOKKED)
		
		p.mo.momx = $/16*17
		p.mo.momy = $/16*17
		p.riu.jumpboost = 1
		if p.mo.eflags & MFE_VERTICALFLIP then
			p.mo.momz = $ + -4*(p.riu.SpinUpFactor)*p.mo.scale * p.riu.wallhitcount
		else
			p.mo.momz = $ + 4*(p.riu.SpinUpFactor)*p.mo.scale * p.riu.wallhitcount
		end
		if (p.riu.wallhitcount <= 0) then
			p.pflags = $ | PF_THOKKED
			p.mo.state = S_PLAY_FALL
			p.mo.momx = $/16
			p.mo.momy = $/16
		end
		
		p.riu.wallhitcount = $ - 1
		if p.powers[pw_super] then
			P_Earthquake(p.mo, p.mo, 125*FRACUNIT)
		end
					
		p.riu.wallhit = 0
	end
	
	if p.mo.state != S_PLAY_JUMP and p.mo.state != S_PLAY_ROLL and p.mo.state != S_PLAY_SPINDASH then
		S_StopSoundByID(p.mo, sfx_spnwnd)
	end
	
	if (p.mo.eflags & MFE_JUSTHITFLOOR) or p.mo.state == S_PLAY_RIDE then
		p.riu.jumpboost = 0
	end
	
	
	if not P_IsObjectOnGround(p.mo) and p.mo.skin != "adventuresonic" and p.powers[pw_carry] != CR_ROLLOUT then
		if (p.cmd.buttons & BT_USE) then
			p.riu.SpinUpFactor = 2
		end
		
		if (p.mo.state == S_PLAY_WALK) or (p.mo.state == S_PLAY_RUN) then
			p.springing = 0
			p.pflags = ($ | PF_JUMPED) & ~(PF_SPINNING|PF_THOKKED)
			p.mo.state = S_PLAY_JUMP
		end
		
	-- Exponential Gravity
		if (p.springing == 0) and p.mo.state ~= S_PLAY_RIDE and p.mo.state ~= S_PLAY_FLY and p.mo.state ~= S_PLAY_GLIDE then
			if not (p.mo.flags & MF_NOGRAVITY) and not (p.mo.eflags & MFE_GOOWATER) then
				if not (p.charability == CA_BOUNCE and p.pflags & PF_THOKKED)
					if (((p.mo.momz < 0 and not (p.mo.eflags & MFE_VERTICALFLIP))
					or (p.mo.momz > 0 and (p.mo.eflags & MFE_VERTICALFLIP))))
						if p.riu.spinwind == 0 then
							if p.mo.state == S_PLAY_JUMP and p.riu.marbledived == 0 then
								S_StartSound(p.mo, sfx_spnwnd)
							end
							p.riu.spinwind = 1
						end
						local gravmul2 = 1 + min((p.riu.airtime * FRACUNIT/400000), FRACUNIT/4)
						local gravmul = 1
						if p.mo.eflags & MFE_UNDERWATER then gravmul2 = (1) end
						if p.mo.eflags & MFE_VERTICALFLIP then gravmul2 = ($ * -1) end
						p.mo.momz = $ - FixedMul(FixedMul(gravity, p.mo.scale), (147*FRACUNIT/100) - FRACUNIT) * gravmul * gravmul2
					else
						if p.riu.jumpboost == 0 then
							if p.mo.eflags & MFE_VERTICALFLIP then p.riu.jumpboostamount = -4 else p.riu.jumpboostamount = 4 end
							p.riu.jumpboost = 1
							p.mo.momz = $ + p.riu.jumpboostamount*FRACUNIT
						end
						p.riu.airtime = 0
						local gravmul2 = 1
						local gravmul = 1
						if p.mo.eflags & MFE_VERTICALFLIP then gravmul = -1 end
						if p.mo.eflags & MFE_UNDERWATER then gravmul2 = ($ * FRACUNIT/2) end
						p.mo.momz = $ - FixedMul(FixedMul(gravity, p.mo.scale), (143*FRACUNIT/100) - FRACUNIT) * gravmul
					end
				end
			end
		end
	end
	
	if CeilingRun then
		if HPU.active == true and p.ceilingrun then
			p.thrustfactor = FixedMul(RIU_DefaultThrustFactor(p)*FRACUNIT, 4*FRACUNIT/5)/FRACUNIT
		end
	end
end)

addHook("MobjDamage", function(mo, inf, src)
	if inf and inf.valid and inf.type == MT_PLAYER then
		if (inf.player.charability == CA_DOUBLEJUMP) and inf.player.pflags & PF_THOKKED then
			-- Regain the ability to double jump
			inf.player.pflags = $ & ~PF_THOKKED
		end
	end
end)


-- Totally not a rebound dash
addHook("MobjMoveBlocked", function(mo)
	local p = mo.player
	if not p.valid then return end
	if p.charability2 == CA2_MELEE then return end
	local spin = p.pflags & PF_SPINNING
	local thok = (p.charability == CA_THOK) and (p.pflags & PF_THOKKED)
	
	if not bounce then return end
	if p.mo.skin != "sonic" then return end
	if p.mo.state ~= S_PLAY_JUMP and p.mo.state ~= S_PLAY_ROLL and p.mo.state ~= S_PLAY_SPINDASH then return end
	if not (p.pflags & PF_JUMPED) then return end
	if p.speed < 10*mo.scale then return end
	if not (p.speed > 2000000) then return end
	
	P_BounceMove(mo)
	
	UM_SpawnThok(p)
			
	local dustcount = 6
	while dustcount > 0
		dustcount = $ - 1
		local dust = P_SpawnMobjFromMobj(p.mo, 0, 0, p.mo.scale * 18, MT_SPINDUST)
		dust.state = S_MINECARTSPARK
		dust.momx = (p.mo.momx * 3) + (P_RandomRange(-20,20) * FRACUNIT)
		dust.momy = (p.mo.momy * 3) + (P_RandomRange(-20,20) * FRACUNIT)
		dust.momz = P_RandomRange(-3,3) * FRACUNIT
		dust.scale = FRACUNIT * P_RandomRange(50,125) / 100
		dust.destscale = 0
		dust.angle = P_RandomRange(0,ANGLE_180)
		dust.fuse = 10
	end

	
	p.riu.savedx = mo.momx
	p.riu.savedy = mo.momy
	p.riu.savedz = mo.momz
	p.riu.WHPrep = -1000
	p.riu.wallhit = 1
	
	S_StartSound(mo, sfx_clackh)
		
end, MT_PLAYER)